
#==============================================================================#
# gruby/graph/part/legend.rb
# $Id: legend.rb,v 1.7 2002/09/22 07:56:00 yuya Exp $
#==============================================================================#

class GRb

  module Graph

    # Legend Class
    class Legend

      # Initialize Method

      def initialize(attribute)
        @attribute = attribute
        @fields    = []

        @spacing   = @attribute.interval
        @vertical  = @attribute.vertical
      end

      # Instance Methods

      def draw(image, box)
        if @vertical
          current = 0

          @fields.each { |field|
            field.draw(image, box.x, box.y + current)
            current += field.dy + @spacing
          }

          return current
        else
          current = box.x

          @fields.each() { |field|
            field.draw(image, current, box.y)
            current += field.dx + @spacing
          }

          return current
        end
      end

      def dx
        if @vertical
          unless @fields.empty?
            return @fields.collect { |field| field.dx }.max
          else
            return 0
          end
        else
          unless @fields.empty?
            current = 0
            @fields.each { |field| current += field.dx + @spacing }
            return current - @spacing
          else
            return 0
          end
        end
      end

      def dy
        if @vertical
          unless @fields.empty?
            current = 0
            @fields.each { |field| current += field.dy + @spacing }
            return current - @spacing
          else
            return 0
          end
        else
          unless @fields.empty?
            return @fields.collect { |field| field.dy }.max
          else
            return 0
          end
        end
      end

      def add(font, text, style = nil)
        @fields << Field.new(font, text, style, @attribute)
      end

      class Field

        def initialize(font, text, marker, attribute)
          @text      = Text.new(font, text)
          @marker    = marker
          @marker_dx = attribute.marker.width
          @marker_dy = attribute.marker.height
          @spacing   = attribute.marker.spacing
        end

        def draw(image, x, y)
          if @marker
            cy = y + self.dy / 2.0
            @marker.draw(image, Box.new(x, cy - @marker_dy / 2.0, @marker_dx, @marker_dy))
            @text.draw2(image, x + @marker_dx + @spacing, cy - @text.dy / 2.0)
          else
            @text.draw2(image, x, y)
          end

        end

        def dx
          if @marker
            return @text.dx + @spacing + @marker_dx
          else
            return @text.dx
          end
        end

        def dy
          if @marker
            return [@text.dy, @marker_dy].max
          else
            return @text.dy
          end
        end

      end

      module Marker

        class Base

          def draw(image, box)
            raise "Don't use this class"
          end

        end

        class Color < Base

          def initialize(fill_color, line_color)
            @fill_color = fill_color
            @line_color = line_color
          end

          def draw(image, box)
            image.rect(box.x, box.y, box.x2, box.y2, @line_color, @fill_color)
          end

        end

        class Line < Base

          def initialize(color)
            @color = color
          end

          def draw(image, box)
            cy = box.y + box.dy / 2
            image.line(box.x, cy, box.x2, cy, @color)
          end

        end

        class LineMarker < Base

          def initialize(color, marker)
            @color  = color
            @marker = marker
          end

          def draw(image, box)
            color = @color.allocate(image)
            cx    = box.x + box.dx / 2
            cy    = box.y + box.dy / 2
            image.line(box.x, cy, box.x2, cy, color)
            @marker.draw(image, cx, cy)
          end

        end

      end

    end

  end

end # GRb

#==============================================================================#
#==============================================================================#
